@@ -12,7 +12,7 @@ from vhh_stc.utils import *
 from vhh_stc.Configuration import Configuration
 import cv2
 import json
-
+import glob
 
 class STC(object):
     """
@@ -82,7 +82,13 @@ class STC(object):
 
         vid_name = shots_np[0][0]
         vid_instance = Video()
-        vid_instance.load(os.path.join(self.config_instance.path_videos, vid_name))
+        if vid_name != -1:
+            vid_instance.load(os.path.join(self.config_instance.path_videos, vid_name))
+        else:
+            # If we are not given a film name, then load the film starting with the max_recall_id
+            films = list(glob.glob(os.path.join(self.config_instance.path_videos, f"{max_recall_id}*.m4v")))
+            assert len(films) == 1
+            vid_instance.load(films[0])
 
         # prepare transformation for cnn model
         preprocess = transforms.Compose([
@@ -101,17 +107,31 @@ class STC(object):
 
         frame_cnt = 0
         results_stc_l = []
-        for i, shot in enumerate(vid_instance.getFramesByShots(shots_np, preprocess_pytorch=preprocess)):
-            if (i >= num_shots):
-                break
 
+        predictions_shot = np.array([], dtype=np.int64)
+        for i, shot in enumerate(vid_instance.getFramesByShots(shots_np, batch_size=self.config_instance.batch_size, preprocess_pytorch=preprocess)):
             shot_tensors = shot["Tensors"]
             shot_id = int(shot["sid"])
             start = int(shot["start"])
             stop = int(shot["end"])
-
+        
             # run classifier
-            class_name, nHits, all_preds_np = self.runModel(model, shot_tensors)
+            predictions = self.runModel(model, shot_tensors)
+
+            # Aggregate predictions over batches in the same shot            
+            predictions_shot = np.concatenate((predictions_shot, predictions))
+
+            if not shot["is_final_batch_in_shot"]:
+                continue
+
+            # Process shots
+            all_preds_np = np.array(predictions_shot).flatten()
+            indices, distr = np.unique(all_preds_np, return_counts=True)
+            tmp_idx = np.zeros(len(self.config_instance.class_names)).astype('int')
+            tmp_idx[indices] = distr
+            idx = tmp_idx.argmax(0)
+            class_name = self.config_instance.class_names[idx]
+            nHits = tmp_idx[idx]
 
             if(self.config_instance.save_raw_results == 1):
                 #print("save intermediate/raw results ... ")
@@ -161,6 +181,8 @@ class STC(object):
             print(str(vid_name) + ";" + str(shot_id) + ";" + str(start) + ";" + str(stop) + ";" + str(class_name))
             results_stc_l.append([str(vid_name) + ";" + str(shot_id) + ";" + str(start) + ";" + str(stop) + ";" + str(class_name)])
 
+            predictions_shot = np.array([], dtype=np.int64)
+
         results_stc_np = np.array(results_stc_l)
 
         # export results
@@ -176,39 +198,20 @@ class STC(object):
                  the number of hits within a shot,
                  frame-based predictions for a whole shot
         """
+        input_batch = Variable(tensor_l)
 
-        input_batch = tensor_l
+        # move the input and model to GPU for speed if available
+        if torch.cuda.is_available():
+            input_batch = input_batch.to('cuda')
+            model.to('cuda')
 
-        # prepare pytorch dataloader
-        dataset = data.TensorDataset(input_batch)  # create your datset
+        model.eval()
+        with torch.no_grad():
+            output = model(input_batch)
+            preds = output.argmax(1, keepdim=True)
+            preds_l = preds.detach().cpu().numpy().flatten()
 
-        inference_dataloader = data.DataLoader(dataset, batch_size=self.config_instance.batch_size, num_workers=2)  # create your dataloader
-
-        preds_l = []
-        for i, inputs in enumerate(inference_dataloader):
-            input_batch = inputs[0]
-            input_batch = Variable(input_batch)
-
-            # move the input and model to GPU for speed if available
-            if torch.cuda.is_available():
-                input_batch = input_batch.to('cuda')
-                model.to('cuda')
-
-            model.eval()
-            with torch.no_grad():
-                output = model(input_batch)
-                preds = output.argmax(1, keepdim=True)
-                preds_l.extend(preds.detach().cpu().numpy().flatten())
-
-        preds_np = np.array(preds_l).flatten()
-        indices, distr = np.unique(preds_np, return_counts=True)
-        tmp_idx = np.zeros(len(self.config_instance.class_names)).astype('int')
-        tmp_idx[indices] = distr
-        idx = tmp_idx.argmax(0)
-        class_name = self.config_instance.class_names[idx]
-        nHits = tmp_idx[idx]
-
-        return class_name, nHits, preds_np
+        return preds_l
 
     def loadSbdResults(self, sbd_results_path):
         """
@@ -221,17 +224,30 @@ class STC(object):
         :return: numpy array holding list of detected shots.
         """
 
-        # open sbd results
-        fp = open(sbd_results_path, 'r')
-        lines = fp.readlines()
-        lines = lines[1:]
+        file_ending = os.path.split(sbd_results_path)[-1].split('.')[-1].lower()
+        if file_ending == "csv":
+            # open sbd results
+            fp = open(sbd_results_path, 'r')
+            lines = fp.readlines()
+            lines = lines[1:]
+
+            lines_n = []
+            for i in range(0, len(lines)):
+                line = lines[i].replace('\n', '')
+                line_split = line.split(';')
+                lines_n.append([line_split[0], os.path.join(line_split[1]), line_split[2], line_split[3]])
+            lines_np = np.array(lines_n)
+        elif file_ending == "json":
+            with open(sbd_results_path, 'r') as file:
+                shots = json.load(file)
+
+            lines_n = []
+            for shot in shots:
+                lines_n.append([-1, shot["shotId"], shot["inPoint"], shot["outPoint"]])
+            lines_np = np.array(lines_n)
+        else:
+            raise ValueError("Unknown filetyp found")
 
-        lines_n = []
-        for i in range(0, len(lines)):
-            line = lines[i].replace('\n', '')
-            line_split = line.split(';')
-            lines_n.append([line_split[0], os.path.join(line_split[1]), line_split[2], line_split[3]])
-        lines_np = np.array(lines_n)
         #print(lines_np.shape)
 
         return lines_np
@@ -290,7 +306,7 @@ class STC(object):
                 tmp_line = tmp_line + ";" + stc_results_np[i][c]
             fp.write(tmp_line + "\n")
 
-    def extract_frames_per_shot(self, shots_per_vid_np=None, number_of_frames=1):
+    def extract_frames_per_shot(self, shots_per_vid_np=None, number_of_frames=1, dst_path="", video_path=""):
         print(f'extract {number_of_frames} frame(s) of each shot...')
 
         if (type(shots_per_vid_np) == None):
@@ -305,9 +321,9 @@ class STC(object):
 
         num_shots = len(shots_np)
 
-        vid_name = shots_np[0][0]
+        vid_name = shots_np[0][0] + ".m4v"
         vid_instance = Video()
-        vid_instance.load(os.path.join(self.config_instance.path_videos, vid_name))
+        vid_instance.load(os.path.join(video_path, vid_name))
 
         for i, shot in enumerate(vid_instance.getFramesByShots(shots_np, preprocess_pytorch=None)):
             if (i >= num_shots):
@@ -319,11 +335,90 @@ class STC(object):
             stop = int(shot["end"])
             shot_type = shots_np[i][4]
 
-            if(int(stop - start) > 2 and (shot_type == 'LS' or shot_type == 'MS')):
+            if(int(stop - start) > number_of_frames): # and (shot_type == 'LS' or shot_type == 'MS' or shot_type == 'CU')):
                 # calculate center image
-                center_pos = int((stop - start) / 2)
-                print(f'extract frame of video \"{vid_name}\" at position: {center_pos} start: {start} end: {stop} shot_id: {shot_id} shot_type: {shot_type}')
+                if(number_of_frames == 1):
+                    center_pos = int((stop - start) / 2)
+                    print(f'extract frame of video \"{vid_name}\" at position: {center_pos} start: {start} end: {stop} shot_id: {shot_id} shot_type: {shot_type}')
+                    name = vid_instance.vidName.split('.')[0] + "_sid_" + str(shot_id) + "_pos_" + str(
+                        start + center_pos)
+                    cv2.imwrite(dst_path + str(name) + "_" + str(shot_type) + ".png", all_shots_np[center_pos])
+                elif(number_of_frames > 1):
+                    diff = int(stop - start)
+                    seq_len = int(diff / number_of_frames)
+                    print("-------")
+                    print(start)
+                    print(stop)
+                    print(diff)
+                    print(seq_len)
 
-                dst_path = "/data/ext/VHH/datasets/vhh_rd_nara_v1/frames/"
-                name = vid_instance.vidName.split('.')[0] + "_sid_" + str(shot_id) + "_pos_" + str(start + center_pos)
-                cv2.imwrite(dst_path + name + ".png", all_shots_np[center_pos])
+                    for p in range(int(seq_len / 2), diff, seq_len):
+                        pos = p
+                        print(pos)
+                        #continue
+                        print(
+                            f'extract frame of video \"{vid_name}\" at position: {pos} start: {start} end: {stop} shot_id: {shot_id} shot_type: {shot_type}')
+                        name = vid_instance.vidName.split('.')[0] + "_sid_" + str(shot_id) + "_pos_" + str(pos + start)
+                        cv2.imwrite(dst_path + name + ".png", all_shots_np[pos])
+
+    def export_shots_as_file(self, shots_np, dst_path="./vhh_mmsi_eval_db_tiny/shots/"):
+        print("export shot as video")
+
+        print(shots_np.shape)
+
+        vid_name = shots_np[0][0]
+        vid_instance = Video()
+        vid_instance.load(self.config_instance.path_videos + "/" + vid_name)
+
+        h = int(vid_instance.height)
+        w = int(vid_instance.width)
+        fps = int(vid_instance.frame_rate)
+
+        print(h)
+        print(w)
+        print(fps)
+
+        for i, data in enumerate(vid_instance.getFramesByShots(shots_np, preprocess_pytorch=None)):
+            frames_per_shots_np = data['Images']
+            shot_id = data['sid']
+            #vid_name = data['video_name']
+            start = data['start']
+            stop = data['end']
+            stc_class = data['stc_class']
+
+            print("######################")
+            print(i)
+            print(i % 32 == 0)
+            print(f'sid: {shot_id}')
+            #print(f'vid_name: {vid_name}')
+            print(f'frames_per_shot: {frames_per_shots_np.shape}')
+            print(f'start: {start}, end: {stop}')
+            print(f'stc_class: {stc_class}')
+
+            if (stc_class == "ELS" or stc_class == "LS" or stc_class == "MS" or stc_class == "CU" or stc_class == "I"):
+            #if (stc_class == "NA" or stc_class == "na"):
+                print("save video! ")
+
+                fourcc = cv2.VideoWriter_fourcc(*'MJPG')
+                out = cv2.VideoWriter(dst_path + "/" + stc_class + "_" + str(i) + ".avi", fourcc, 12, (w, h))
+
+                font = cv2.FONT_HERSHEY_SIMPLEX
+                bottomLeftCornerOfText = (30, 50)
+                fontScale = 1
+                fontColor = (255, 255, 255)
+                lineType = 2
+
+                for j in range(0, len(frames_per_shots_np)):
+                    frame = frames_per_shots_np[j]
+                    cv2.rectangle(frame, (0, 0), (350, 80), (0, 0, 255), -1)
+                    cv2.putText(frame, "Shot-Type: " + stc_class,
+                                bottomLeftCornerOfText,
+                                font,
+                                fontScale,
+                                fontColor,
+                                lineType)
+                    out.write(frame)
+                    #cv2.imshow("test", frame)
+                    #k = cv2.waitKey(10)
+
+                out.release()
